from abc import ABC, abstractmethod
from logging import INFO, getLogger

from user_agent import generate_user_agent

logger = getLogger()
logger.setLevel(INFO)


class RequestBaseDownloader(ABC):
    def __init__(self, header: dict = None, proxy: dict = None):
        self._header = header or self.custom_header
        self.proxies = proxy

    @abstractmethod
    def get(self, *args, **kwargs):
        """Need to be implemented"""

    @abstractmethod
    def post(self, *args, **kwargs):
        """Need to be implemented"""

    @property
    def header(self):
        return self._header

    @header.setter
    def header(self, headers: dict):
        self._header = headers

    @property
    def custom_header(self):
        return {
            "User-Agent": (
                "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 "
                "(KHTML, like Gecko) Chrome/84.0.4147.125 "
                "Safari/537.36"
            ),
        }

    @property
    def random_user_agent(self):
        return generate_user_agent()



import logging
import asyncio

from .adapters.specs.http import *
from .adapters.asynchronous import *
from .commands.action import *
from .commands.download import *
from .core.asynchronous import *
from .core.base import *

fmt = "[%(asctime)s] [%(filename)s: %(lineno)d] [%(levelname)s] => %(message)s"

logging.basicConfig(
    level="INFO", format=fmt,
)



import asyncio


async def gather_with_concurrency(n, *tasks):
    semaphore = asyncio.Semaphore(n)

    async def sem_task(task):
        async with semaphore:
            return await task

    return await asyncio.gather(*(sem_task(task) for task in tasks))



import os
from logging import getLogger
from pathlib import Path
from time import sleep
from typing import List, Union

from selenium.common.exceptions import TimeoutException
from selenium.webdriver import ActionChains, Chrome
from selenium.webdriver.android.webdriver import WebDriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support import expected_conditions as ec
from selenium.webdriver.support.ui import WebDriverWait


logger = getLogger()


class ChromeDriver:
    def __init__(self):
        self.driver = self.__start_chrome_driver()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.__close()

    def __del__(self):
        self.__close()

    def __close(self):
        try:
            self.driver.close()
        except Exception:
            pass

    def find_element(
        self, select: str, strategy, driver_or_element=None, first=True
    ) -> Union[List[WebElement], WebElement]:
        """
        It will find a element in the driver, following the strategy available on this module
        `downloader.adapters.strategy`
        Could be -> FindByXpath, FindById, FindByCss, FindByName, FindByClassName, FindByTagName
        :param select: String
        :param strategy: downloader.adapters.strategy
        :param driver_or_element: use an specific element or the properly driver
        :param first: Bool
        :return: List of elements, or first element, or None
        """

        driver_or_element = driver_or_element or self.driver

        return strategy.execute(select, driver_or_element, first)

    def expect_presence_of_element(
        self, select: str, strategy: By, max_wait_time: int = 5
    ) -> Union[WebElement, None]:
        """
        This method expect some condition or element exists to be truth and continue.
        To know more, see this article
        https://selenium-python.readthedocs.io/waits.html#explicit-waits
        :param select: STRING -> A selector in XPATH, ID, CSS, etc.
        :param strategy: selenium.webdriver.common.by.BY ->
                            A strategy to find some element, like:
                            By.XPATH, By.ID, etc
        :param max_wait_time: INT
        :return: WebElement
        """
        try:
            return WebDriverWait(self.driver, max_wait_time).until(
                ec.presence_of_element_located((strategy, select))
            )
        except TimeoutException:
            return None

    @staticmethod
    def fill(element, value: str, submit=False):
        element.send_keys(value)
        if submit:
            element.submit()

    def move_mouse_to_element(self, element):
        try:
            action = ActionChains(self.driver)
            action.move_to_element(element)
            action.perform()
        except Exception as err:
            logger.error(
                f"Couldn't move to element - {type(err)} - {err}"
            )

    def scroll_until_end_page(self, SCROLL_PAUSE_TIME=0.3, SCROLL_VALUE=500):
        SCROLL_PAUSE_TIME = SCROLL_PAUSE_TIME

        # Get scroll height
        last_height = self.driver.execute_script("return document.body.scrollHeight;")

        x = 0
        y = SCROLL_VALUE

        SCROLL_VALUE = SCROLL_VALUE

        self.driver.execute_script("window.scrollTo(0, 0);")

        while y < last_height:
            # Scroll down to bottom
            self.driver.execute_script(f"window.scrollTo({x},{y});")

            x += SCROLL_VALUE
            y += SCROLL_VALUE

            # Wait to load page
            sleep(SCROLL_PAUSE_TIME)

    @staticmethod
    def __start_chrome_driver() -> WebDriver:
        chrome_options = Options()
        chrome_options.add_argument("--start-maximized")
        # chrome_options.add_argument("--headless")
        # chrome_options.add_argument("--no-sandbox")
        # chrome_options.add_argument("--disable-dev-shm-usage")

        try:
            parent_path = Path(os.getcwd()).parent
            path = Path(parent_path).parent
            return Chrome(f"{path}/chromedriver", options=chrome_options)
        except (ImportError, FileNotFoundError):
            return Chrome("chromedriver", options=chrome_options)



from dataclasses import dataclass
from typing import Optional, Union

from eiprice.httpclient.core.methods import Method


@dataclass
class DownloadSpec:
    url: str
    method: Method
    headers: Optional[dict] = None
    params: Optional[dict] = None
    json: Optional[dict] = None
    data: Optional[Union[dict, str]] = None
    proxies: Optional[dict] = None
    time_between_requests: Union[int, float] = None
    verify: Optional[bool] = True
    timeout: Optional[int] = 60




from time import sleep
from logging import INFO, getLogger
from typing import Union

import backoff
from requests_html import HTMLSession

from eiprice.httpclient.core.base import RequestBaseDownloader
from eiprice.httpclient.core.methods import Method

logger = getLogger()
logger.setLevel(INFO)


class RequestSyncDownloader(RequestBaseDownloader):
    def __init__(self, header: dict = None, proxy: dict = None):
        super().__init__(header, proxy)
        self.session = HTMLSession()

    def __del__(self):
        try:
            self.session.close()
        except Exception as err:
            logger.error(
                "Something went wrong trying to close session. "
                f"{type(err)} - {err}"
            )

    @property
    def cookies(self):
        return self.session.cookies

    def get(self, url, *args, **kwargs):
        """
        Example:
            payload = {'some': 'data'}
            headers = {'content-type': 'application/json'}
            params = {'key1': 'value1', 'key2': 'value2'}
        """
        return self.execute("get", url, *args, **kwargs)

    def post(self, url, *args, **kwargs):
        """
        Example:
            payload = {'some': 'data'}
            headers = {'content-type': 'application/json'}
            params = {'key1': 'value1', 'key2': 'value2'}
        """
        return self.execute("post", url, *args, **kwargs)

    @backoff.on_exception(
        backoff.expo, Exception, max_tries=3, max_time=30,
    )
    def execute(
        self,
        method: str,
        url,
        headers: dict = None,
        json: dict = None,
        params: dict = None,
        cookies=None,
        payload=None,
        files=None,
        timeout: int = 60,
        proxies: dict = None,
        time_between_requests: Union[int, float] = None,
        verify=True,
    ):
        if method.lower() not in [method.value for method in Method]:
            raise ValueError

        logger.info(f"{method} - {url}")
        options = {
            "headers": headers if headers else self.custom_header,
            "verify": verify,
        }

        if time_between_requests:
            sleep(time_between_requests)

        if payload:
            options["data"] = payload
        if json:
            options["json"] = json
        if params:
            options["params"] = params
        if cookies:
            options["cookies"] = cookies
        if files:
            options["files"] = files
        if timeout:
            options["timeout"] = timeout
        if proxies:
            options["proxies"] = proxies

        return self.session.request(method, url, **options)



import setuptools
import os
from setuptools import find_packages

dependencies = [
    'Unidecode',
    'nest_asyncio',
    'backoff',
    'requests-html',
    'user_agent'
]

setuptools.setup(
    name='eiprice',
    version='2.0.1',
    packages=find_packages(),
    install_requires=dependencies,
)



# import asyncio
from typing import List

# import nest_asyncio

from eiprice.httpclient.adapters.asynchronous import gather_with_concurrency
from eiprice.httpclient.adapters.specs.http import DownloadSpec
from eiprice.httpclient.commands.action import Action
# from httpclient.core.asynchronous import RequestAsyncDownloader
from eiprice.httpclient.core.synchronous import RequestSyncDownloader

# nest_asyncio.apply()


# def exec_async_coroutine(coroutines):
#     loop = asyncio.get_event_loop()
#     result = loop.run_until_complete(coroutines)

#     return result


class SyncDownloadSingleURL(Action):
    def __init__(self, spec: DownloadSpec):
        self.spec = spec
        self.downloader = RequestSyncDownloader()

    def execute(self):
        return self.downloader.execute(
            self.spec.method.value,
            self.spec.url,
            headers=self.spec.headers,
            params=self.spec.params,
            json=self.spec.json,
            payload=self.spec.data,
            proxies=self.spec.proxies,
            verify=self.spec.verify,
            timeout=self.spec.timeout,

        )


# class AsyncDownloadSingleURL(Action):
#     def __init__(self, spec: DownloadSpec):
#         self.spec = spec
#         self.downloader = RequestAsyncDownloader()

#     def execute(self):
#         result = exec_async_coroutine(
#             self.downloader.execute(
#                 self.spec.method.value,
#                 self.spec.url,
#                 headers=self.spec.headers,
#                 params=self.spec.params,
#                 json=self.spec.json,
#                 payload=self.spec.data,
#                 proxies=self.spec.proxies,
#             )
#         )

#         return result


# class AsyncDownloadManyURLS(Action):
#     def __init__(self, specs: List[DownloadSpec], max_workers: int = 5):
#         self.specs = specs
#         self.max_workers = max_workers
#         self.downloader = RequestAsyncDownloader()

#     def execute(self):
#         result = exec_async_coroutine(
#             gather_with_concurrency(
#                 self.max_workers,
#                 *[
#                     self.downloader.execute(
#                         spec.method.value,
#                         spec.url,
#                         headers=spec.headers,
#                         params=spec.params,
#                         json=spec.json,
#                         payload=spec.data,
#                         proxies=spec.proxies,
#                         time_between_requests=spec.time_between_requests,
#                     )
#                     for spec in self.specs
#                 ]
#             )
#         )

#         return result



from abc import ABC, abstractmethod
from functools import wraps

from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.chrome.webdriver import WebDriver


def shield_from_no_such_element(func):
    @wraps(func)
    def decorated_view(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except NoSuchElementException:
            return None

    return decorated_view


class ContentFinderStrategy(ABC):
    @abstractmethod
    def execute(self, select, driver, first=True):
        pass


class FindByXpath(ContentFinderStrategy):
    @staticmethod
    @shield_from_no_such_element
    def execute(select: str, driver: WebDriver, first=True):
        if first:
            return driver.find_element_by_xpath(select)
        return driver.find_elements_by_xpath(select)


class FindById(ContentFinderStrategy):
    @staticmethod
    @shield_from_no_such_element
    def execute(select: str, driver: WebDriver, first=True):
        if first:
            return driver.find_element_by_id(select)
        return driver.find_elements_by_id(select)


class FindByCss(ContentFinderStrategy):
    @staticmethod
    @shield_from_no_such_element
    def execute(select: str, driver: WebDriver, first=True):
        if first:
            return driver.find_element_by_css_selector(select)
        return driver.find_elements_by_css_selector(select)


class FindByName(ContentFinderStrategy):
    @staticmethod
    @shield_from_no_such_element
    def execute(select: str, driver: WebDriver, first=True):
        if first:
            return driver.find_element_by_name(select)
        return driver.find_elements_by_name(select)


class FindByTagName(ContentFinderStrategy):
    @staticmethod
    @shield_from_no_such_element
    def execute(select: str, driver: WebDriver, first=True):
        if first:
            return driver.find_element_by_tag_name(select)
        return driver.find_elements_by_tag_name(select)


class FindByClassName(ContentFinderStrategy):
    @staticmethod
    @shield_from_no_such_element
    def execute(select: str, driver: WebDriver, first=True):
        if first:
            return driver.find_element_by_class_name(select)
        return driver.find_elements_by_class_name(select)
